Attribute VB_Name = "modLevelHandler"
Option Explicit

Type lvlObject
  X As Integer
  Y As Integer
  Width As Integer
  Height As Integer
  Tile1 As Byte
  Tile2 As Byte
  DispColour As Long
End Type

Public ObjectCount As Integer
Public LevelDispName As String
Public LevelFileNum As Long
Public LevelHeaderNum As Long
Public CurrentEdited As Integer
Public lvlObjects() As lvlObject
Public lvlHeaderWork() As Byte
Public Const conDefColour As Integer = 15
Public Const conDefSelColour As Integer = 12
Public Const ObjDefIni As String = "\objdef.ini"

Public Sub LoadLevel(FileNum As Long, DisplayName As String)
  Dim LevelSize As Long, Progress As Integer, tProgress As Integer
  Dim TempStr As String
  
  LevelSize = ntrFiles(FileNum).End - ntrFiles(FileNum).Start - 1
  ObjectCount = (LevelSize - 2) / 10
  LevelDispName = DisplayName
  LevelFileNum = FileNum
  frmLevelEdit.lstObjects.Clear
  
  If Not ntrQuickFindFile.Exists(Replace(ntrFiles(FileNum).Name, "_bgdat", "")) Then
    MsgBox "Your ROM is most likely corrupted.. The level header file for this level could not be found. Please try again using a clean ROM!", vbCritical Or vbOKOnly
    End
  End If
  
  LevelHeaderNum = ntrQuickFindFile(Replace(ntrFiles(FileNum).Name, "_bgdat", ""))
  lvlHeaderWork = ntrFiles(LevelHeaderNum).Contents
  ReDim lvlObjects(ObjectCount)
  
  For Progress = 1 To ObjectCount
    tProgress = (Progress - 1) * 10
    lvlObjects(Progress).Tile1 = ntrFiles(FileNum).Contents(tProgress)
    lvlObjects(Progress).Tile2 = ntrFiles(FileNum).Contents(tProgress + 1)
    lvlObjects(Progress).X = CLng(ntrFiles(FileNum).Contents(tProgress + 2)) + (CLng(ntrFiles(FileNum).Contents(tProgress + 3)) * 256)
    lvlObjects(Progress).Y = CLng(ntrFiles(FileNum).Contents(tProgress + 4)) + (CLng(ntrFiles(FileNum).Contents(tProgress + 5)) * 256)
    lvlObjects(Progress).Width = CLng(ntrFiles(FileNum).Contents(tProgress + 6)) + (CLng(ntrFiles(FileNum).Contents(tProgress + 7)) * 256)
    lvlObjects(Progress).Height = CLng(ntrFiles(FileNum).Contents(tProgress + 8)) + (CLng(ntrFiles(FileNum).Contents(tProgress + 9)) * 256)
    TempStr = ReadIniValue(App.Path & ObjDefIni, Trim(Str(ntrFiles(FileNum).Contents(tProgress + 1))), Trim(Str(ntrFiles(FileNum).Contents(tProgress))) & "colour")
    If TempStr = "ERROR" Then
      TempStr = ReadIniValue(App.Path & ObjDefIni, Trim(Str(ntrFiles(FileNum).Contents(tProgress + 1))), "sectcolour")
      If TempStr = "ERROR" Then
        lvlObjects(Progress).DispColour = QBColor(conDefColour)
      Else
        If Val(TempStr) = 15 Then
          lvlObjects(Progress).DispColour = frmLevelEdit.pbxDisplay.BackColor
        Else
          lvlObjects(Progress).DispColour = QBColor(Val(TempStr))
        End If
      End If
    Else
      If Val(TempStr) = 15 Then
        lvlObjects(Progress).DispColour = frmLevelEdit.pbxDisplay.BackColor
      Else
        lvlObjects(Progress).DispColour = QBColor(Val(TempStr))
      End If
    End If
    frmLevelEdit.lstObjects.AddItem AssembleLogString(Progress)
  Next Progress
End Sub

Public Sub RedrawLevel()
  Dim Progress As Integer
  
  frmLevelEdit.pbxDisplay.Cls
  For Progress = 1 To ObjectCount
    frmLevelEdit.DrawObject Progress, False
    If frmLevelEdit.lstObjects.ListIndex = Progress - 1 Then
      frmLevelEdit.DrawSelObject Progress
    End If
  Next Progress
End Sub

Public Sub OpenLevel()
  If frmLevelEdit.HasModified Then
    Select Case MsgBox("Your level contains unsaved changes." _
           & vbCrLf & "Would you like to save them before opening another level? :)" _
           , vbYesNoCancel Or vbExclamation Or vbDefaultButton1, App.Title)
      Case vbYes
        SaveLevel
      Case vbCancel
        Exit Sub
    End Select
  End If
  
  frmLevelEdit.Hide
  frmEditTile.Hide
  frmPickTile.Hide
  frmHeaderEdit.Hide
  frmLoadROM.Show
End Sub

Public Sub ReloadLevel()
  If frmLevelEdit.HasModified Then
    If MsgBox("Your level contains unsaved changes." _
           & vbCrLf & "Are you sure you want to discard them?" _
           , vbYesNo Or vbExclamation Or vbDefaultButton1, App.Title) _
      = vbNo Then Exit Sub
  End If
  
  LoadLevel LevelFileNum, LevelDispName
  frmEditTile.Hide
  frmPickTile.Hide
  frmHeaderEdit.Hide
  frmLevelEdit.InitLevel
End Sub

Public Sub SaveLevel()
  Dim Progress As Integer, hnd As Integer, TempStr As String
  
  If Dir(ROMPath) = "" Then
    MsgBox "The ROM has been renamed, moved or deleted since it was opened in NSMB Editor." & vbLf _
         & "The level cannot be saved.", vbCritical Or vbOKOnly
    Exit Sub
  End If
  
  hnd = FreeFile
  Open ROMPath For Binary As #hnd
    'Save the level
    Seek #hnd, ntrFiles(LevelFileNum).Start + 1
    For Progress = 1 To ObjectCount
      'Assemble the levelfile string
      TempStr = Chr(lvlObjects(Progress).Tile1)
      TempStr = TempStr & Chr(lvlObjects(Progress).Tile2)
      TempStr = TempStr & Int2BToStr(lvlObjects(Progress).X)
      TempStr = TempStr & Int2BToStr(lvlObjects(Progress).Y)
      TempStr = TempStr & Int2BToStr(lvlObjects(Progress).Width)
      TempStr = TempStr & Int2BToStr(lvlObjects(Progress).Height)
      Put #hnd, , TempStr
    Next Progress
    TempStr = Chr(255) & Chr(255)
    Put #hnd, , TempStr
  
    'Let's put the updated level back in ntrFiles.Contents
    Seek #hnd, ntrFiles(LevelFileNum).Start + 1
    Get #hnd, , ntrFiles(LevelFileNum).Contents
  
    'Next, save the level header
    Seek #hnd, ntrFiles(LevelHeaderNum).Start + 1
    Put #hnd, , lvlHeaderWork
    
    'And let's put the updated header back in ntrFiles.Contents
    ntrFiles(LevelHeaderNum).Contents = lvlHeaderWork
  Close #hnd

  MsgBox "Successfully saved level " & ntrFiles(LevelFileNum).Name & " with " & Trim(Str(ObjectCount)) & " objects!", vbInformation Or vbOKOnly
End Sub

Public Sub EditObject(ObjNum As Integer)
  CurrentEdited = ObjNum
  frmEditTile.Show
  frmEditTile.GetReady
End Sub

Public Function AssembleLogString(ObjNum As Integer) As String
  Dim TempStr As String, TempStr2 As String
  
  TempStr = "(" & Trim(Str(lvlObjects(ObjNum).X))
  TempStr = TempStr & "," & Trim(Str(lvlObjects(ObjNum).Y))
  TempStr = TempStr & " " & Trim(Str(lvlObjects(ObjNum).Width))
  TempStr = TempStr & "x" & Trim(Str(lvlObjects(ObjNum).Height))
  TempStr2 = ReadIniValue(App.Path & ObjDefIni, Trim(Str(lvlObjects(ObjNum).Tile2)), Trim(Str(lvlObjects(ObjNum).Tile1)))
  If TempStr2 = "ERROR" Then TempStr2 = "Unknown Tile"
  TempStr = TempStr & ") " & TempStr2
  TempStr = TempStr & " (" & Trim(Str(lvlObjects(ObjNum).Tile1))
  TempStr = TempStr & " " & Trim(Str(lvlObjects(ObjNum).Tile2))
  TempStr = TempStr & ")"
  
  AssembleLogString = TempStr
End Function

Public Sub UpdateStatus()
  Dim TempStr As String
  
  TempStr = LevelDispName & " at offset 0x" & Hex(ntrFiles(LevelFileNum).Start)
  TempStr = TempStr & " - " & Trim(Str(ObjectCount)) & " objects"
  
  frmLevelEdit.stbStatus.SimpleText = TempStr
End Sub
